unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Runner, ExtCtrls;

type
  TfrmMain = class(TForm)
    lbRunning: TListBox;
    lbStopped: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    OpenDialog1: TOpenDialog;
    btnRun: TButton;
    btnStop: TButton;
    btnTerminate: TButton;
    Runner1: TRunner;
    Bevel1: TBevel;
    btnMinimize: TButton;
    btnMaximize: TButton;
    Bevel2: TBevel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    lblStarted: TLabel;
    lblRunning: TLabel;
    lblUser: TLabel;
    lblKernel: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    Timer1: TTimer;
    Label9: TLabel;
    btnBringToFront: TButton;
    procedure btnRunClick(Sender: TObject);
    procedure Runner1AppTerminate(App: TAppInfo);
    procedure Runner1AppIdle(App: TAppInfo);
    procedure btnStopClick(Sender: TObject);
    procedure btnTerminateClick(Sender: TObject);
    procedure btnMinimizeClick(Sender: TObject);
    procedure btnMaximizeClick(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnBringToFrontClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

  TAppObject = class
    wH: THandle;
    pH: THandle;
    tH: THandle;
    pID: DWORD;
    tID: DWORD;
  end;

var
  frmMain: TfrmMain;
  AppObject: TAppObject;

implementation

{$R *.DFM}

procedure TfrmMain.btnRunClick(Sender: TObject);
var
  sFile: String;

begin
  if OpenDialog1.Execute then
  begin
    sFile := OpenDialog1.FileName;
    Runner1.FileToRun := sFile;
    Runner1.Run;
    sFile := ExtractFileName(sFile);
    AppObject := TAppObject.Create;
    with AppObject, Runner1 do
    begin
      pH := ProcessHandle;
      tH := ThreadHandle;
      pID := ProcessID;
      tID := ThreadID;
    end;
    lbRunning.Items.AddObject(sFile, AppObject);
  end;
end;

procedure TfrmMain.Runner1AppTerminate(App: TAppInfo);
var
  sFile: String;
  oTemp: TAppObject;

begin
  sFile := ExtractFileName(App.Name);
  lbStopped.Items.Add(sFile);
  oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.Items.IndexOf(sFile)]);
  oTemp.Free;
  lbRunning.Items.Delete(lbRunning.Items.IndexOf(sFile));
  ShowMessage('''' + sFile + '''' + #13#10 +
              'Started at: ' + DateTimeToStr(App.Started) + #13#10 +
              'Ended at: ' + DateTimeToStr(App.Ended) + #13#10 +
              'Kernel time: ' + FloatToStr(App.KernelTime * 100) + ' nanoseconds' + #13#10 +
              'User time: ' + FloatToStr(App.UserTime * 100) + ' nanoseconds');
end;

procedure TfrmMain.Runner1AppIdle(App: TAppInfo);
var
  sFile: String;
  oTemp: TAppObject;

begin
  sFile := ExtractFileName(App.Name);
  oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.Items.IndexOf(sFile)]);
  oTemp.wH := Runner1.GetHandle;
end;

procedure TfrmMain.btnStopClick(Sender: TObject);
var
  oTemp: TAppObject;

begin
  if lbRunning.ItemIndex > -1 then
  begin
    oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.ItemIndex]);
    PostMessage(oTemp.wH, WM_CLOSE, 0, 0);
  end
  else
    ShowMessage('Select a ''running'' file to stop');
end;

procedure TfrmMain.btnTerminateClick(Sender: TObject);
var
  oTemp: TAppObject;

begin
  if lbRunning.ItemIndex > -1 then
  begin
    oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.ItemIndex]);
    TerminateProcess(oTemp.pH, 0);
  end
  else
    ShowMessage('Select a ''running'' file to terminate');
end;

procedure TfrmMain.btnMinimizeClick(Sender: TObject);
var
  oTemp: TAppObject;

begin
  if lbRunning.ItemIndex > -1 then
  begin
    oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.ItemIndex]);
    ShowWindow(oTemp.wH, SW_MINIMIZE);
  end
  else
    ShowMessage('Select a ''running'' file to minimize');
end;

procedure TfrmMain.btnMaximizeClick(Sender: TObject);
var
  oTemp: TAppObject;

begin
  if lbRunning.ItemIndex > -1 then
  begin
    oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.ItemIndex]);
    ShowWindow(oTemp.wH, SW_MAXIMIZE);
  end
  else
    ShowMessage('Select a ''running'' file to maximize');
end;


procedure TfrmMain.Timer1Timer(Sender: TObject);
var
  Started: TDateTime;
  Kernel, User: Comp;

begin
  if Runner1.GetTimes(Started, Kernel, User) then
  begin
    lblStarted.Caption := DateTimeToStr(Started);
    lblRunning.Caption := TimeToStr(Now - Started);
    lblKernel.Caption := FloatToStr(Kernel * 100);
    lblUser.Caption := FloatToStr(User * 100);
  end;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Application.Title := '#7 Component - TRunner Demo';
end;

procedure TfrmMain.btnBringToFrontClick(Sender: TObject);
var
  oTemp: TAppObject;

begin
  if lbRunning.ItemIndex > -1 then
  begin
    oTemp := TAppObject(lbRunning.Items.Objects[lbRunning.ItemIndex]);
    BringWindowToTop(oTemp.wH);
  end
  else
    ShowMessage('Select a ''running'' file to bring it''s window to front');
end;

end.
